/**
 * Stars!
 * 星間国家クラス
 *
 * $Id: stars_empire.js,v 1.13 2006/02/13 14:37:12 shio Exp $
 */

// 星間国家クラス
function Empire(id, name, color, fnd, nvg, mlt, trs, umode, fnd_p, nvg_p, mlt_p) {
    this.id = id;           // 識別番号(1 ～)
    this.name = name;       // 名称
    this.color = color;     // 色("#FFFFFF"形式)
    this.fnd = fnd;         // 探査技術
    this.nvg = nvg;         // 航宙技術
    this.mlt = mlt;         // 軍事技術
    this.trs = trs;         // 国庫
    this.fnd_p = (fnd_p) ? fnd_p : 0; // 探査技術開発ポイント
    this.nvg_p = (nvg_p) ? nvg_p : 0; // 航宙技術開発ポイント
    this.mlt_p = (mlt_p) ? mlt_p : 0; // 軍事技術開発ポイント
    this.foundPlanetList = new Array();
    this.ctlPlanetList = new Array();
    this.otherPlanetList = new Array();
    this.yoronList = new Object();
    this.fleet = new Array();
    this.tmp_trs = trs;
    this.user_mode = umode; // ユーザー支配かどうか
    this.visibleList = new Object();
    this.settlerList = new Array();
    this.kaisanFleet = new Array();
    
    if (umode) {
        this.ai = new UserAgent(app, this);
        this.ai.empire = this;
    } else {
        this.ai = new Ai(app, this);
        this.ai.empire = this;
    }
    
    /* 移民船団追加 */
    this.addSettler = function(loc) {
        this.settlerList.push(loc);
    }
    
    /* 移民船団除去 */
    this.removeSettler = function(loc) {
        var newList = new Array();
        for (var i = 0; i < this.settlerList.length; i++) {
            if (this.settlerList[i] != loc) {
                newList.push(this.settlerList[i]);
            }
        }
        this.settlerList = newList;
    }
    
    /* 移民船団移動 */
    this.moveSettler = function(fromloc, toloc) {
        for (var i = 0; i < this.settlerList.length; i++) {
            if (this.settlerList[i] == fromloc) {
                this.settlerList[i] = toloc;
                break;
            }
        }
        this.makeSettlerVisibleList();
    }

    this.buildFleet = function(planet) {
        var u = new FleetUnit(this, 0, this.nvg, this.mlt, planet.id, 0, 0);
        this.fleet.push(u);
    }
    
    this.addFleet = function(u) {
        this.fleet.push(u);
    }
    
    this.removeFleet = function(unit) {
        var newFleet = new Array();
        for (var i = 0; i < this.fleet.length; i++) {
            if (this.fleet[i].no != unit.no) {
                newFleet.push(this.fleet[i]);
            }
        }
        this.fleet = newFleet;
   }

    this.countFleet = function() {
        return this.fleet.length;
    }

    this.fleetNeeds = function() {
        result = Math.floor(this.totalEcn(this.ctlPlanetList) / 15);
        return result;
    }

    this.addKaisanFleet = function(unit) {
        this.kaisanFleet.push(unit);
    }

    this.removeKaisanFleet = function(unit) {
        var newList = new Array();
        for (var ix = 0; ix < this.kaisanFleet.length; ix++) {
            if (unit.no != this.kaisanFleet[ix].no) {
                newList.push(this.kaisanFleet[ix]);
            }
        }
        this.kaisanFleet = newList;
    }

    this.executeKaisanList = function() {
        var msg = "";
        for (var ix = 0; ix < this.kaisanFleet.length; ix++) {
            msg += this.kaisanFleet[ix].getName() + "を解散しました。<br/>";
            this.removeFleet(this.kaisanFleet[ix]);
        }
        if (msg != "") {
            printConsole(this.id, msg);
        }
        this.kaisanFleet = new Array();
    }

    this.isContainsKaisanList = function(no) {
        for (var ix = 0; ix < this.kaisanFleet.length; ix++) {
            if (this.kaisanFleet[ix].no == no) {
                return true;
            }
        }
        return false;
    }

    this.getFleetByPlanetId = function(pid) {
        for (var ix = 0; ix < this.fleet.length; ix++) {
            if (this.fleet[ix].pid == pid) {
                return this.fleet[ix];
            }
        }
        return null;
    }
    
    this.getKaisanFleetByPlanetId = function(pid) {
        for (var ix = 0; ix < this.kaisanFleet.length; ix++) {
            if (this.kaisanFleet[ix].pid == pid) {
                return this.kaisanFleet[ix];
            }
        }
        return null;
    }

    this.getFleetByNo = function(no) {
        for (var ix = 0; ix < this.fleet.length; ix++) {
            if (this.fleet[ix].no == no) {
                return this.fleet[ix];
            }
        }
        return null;
    }
    
    this.getFleetNo = function() {
        var hash = new Object();
        for (var ix = 0; ix < this.fleet.length; ix++) {
            var u = this.fleet[ix];
            if (u) {
                hash[u.no] = true;
            }
       }
        for (var i = 1; true; i++) {
            if (! hash[i]) {
                return i;
            }
        }
    }

    this.updateFleetByTurn = function() {
        for (var ix = 0; ix < this.fleet.length; ix++) {
            this.fleet[ix].updateByTurn();
        }
    }

    this.unctl = function(p) {
        var list = new Array();
        for (var i = 0; i < this.ctlPlanetList.length; i++) {
            if (p.id == this.ctlPlanetList[i].id) {
            } else {
                list.push(this.ctlPlanetList[i]);
            }
        }
        this.ctlPlanetList = list;
    }

    this.newctl = function(p) {
        this.ctlPlanetList.push(p);
        this.makeVisibleList();
    }

    // 所属惑星リストを作成する
    this.makePlanetList = function(starList) {
        var result = new Array();
        for (var lk in starList) {
            var p = starList[lk];
            if ( (p.ctl == this) && (p.dpl > 0) ) {
                result.push(p);
            }
        }

        return result;
    }

    // 可視リストを作成する
   this.makeVisibleList = function() {
        var starList = this.ctlPlanetList;
        var result = new Object();
        if (! starList) {
            return result;
        }
        for (var row = 0; row < app.height; row++) {
            for (var col = 0; col < app.width; col++) {
                var lk = makeLocationKey(row, col);
                if (this.visibleList[lk]) {
                    continue;
                }
                for (var i = 0; i < starList.length; i++) {
                    var p = starList[i];
                    if (calDistance(p.loc, lk) <= this.fnd) {
                        this.visibleList[lk] = true;
                        break;
                    }
                }
                if (this.visibleList[lk]) {
                    continue;
                }
                for (var i = 0; i < this.settlerList.length; i++) {
                    if (calDistance(this.settlerList[i], lk) <= this.fnd) {
                        this.visibleList[lk] = true;
                        break;
                    }
                }
            }
        }
    }

    // 移民船団移動時に可視リストを更新する。
    this.makeSettlerVisibleList = function() {
        var starList = this.ctlPlanetList;
        var result = new Object();
        if (! starList) {
            return result;
        }
        for (var row = 0; row < app.height; row++) {
            for (var col = 0; col < app.width; col++) {
                var lk = makeLocationKey(row, col);
                if (this.visibleList[lk]) {
                    continue;
                }
                for (var i = 0; i < this.settlerList.length; i++) {
                    if (calDistance(this.settlerList[i], lk) <= this.fnd) {
                        this.visibleList[lk] = true;
                        break;
                    }
                }
            }
        }
    }

    // 表示可能かどうかを判断する
    this.isVisible = function(loc) {
        if (! this.visibleList) {
            return false;
        }
        if (this.visibleList[loc]) {
            return true;
        }
        
        for (var i = 0; i < app.empireList.length; i++) {
            var e = app.empireList[i];
            if (this.yoronList[e.id] ) {
                if (e && e.visibleList[loc]) {
                    return true;
                }
            }
        }

        return false;
    }

    // 経済力を算出する
    this.totalEcn = function() {
        var result = 0;
        var pList = this.ctlPlanetList;
        for (var i = 0; i < pList.length; i++) {
            var p = pList[i];
            result += p.ecn;
        }

        return result;
    }

    // 総人口を算出する
    this.totalDpl = function() {
        var result = 0;
        var pList = this.ctlPlanetList;
        for (var i = 0; i < pList.length; i++) {
            var p = pList[i];
            result += p.dpl;
        }

        return result;
    }

    // 支配リストを作成する
    this.makeCtlList = function() {
        this.ctlPlanetList = new Array();
        for (var loc in app.starList) {
            var p = app.starList[loc];
            if (p.ctl) {
                if (p.ctl == this) {
                    this.ctlPlanetList.push(p);
                }
            }
        }
        this.makeVisibleList();
    }

    this.getNearestPlanetDistance = function(lk) {
        var min = 999;
        for (var i = 0; i < this.ctlPlanetList.length; i++) {
            var p = this.ctlPlanetList[i];
            var d = calDistance(p.loc, lk);
            if (d < min) {
                min = d;
            }
        }
        return min;
    }

    // 参加可能な艦隊リストを取得する。
    this.getReachedFleetList = function(tloc) {
        var msg = "";
        var list = new Array();
        for (var i = 0; i < this.fleet.length; i++) {
            if (this.fleet[i].pid) {
                var planet = app.getPlanetById(this.fleet[i].pid);
                if (this.ai.checkBetweenNavigatable(
                    tloc, planet.loc, this.fleet[i].nvg, this.ctlPlanetList) ) {
                    list.push(this.fleet[i]);
                    msg += this.fleet[i].getName() + "は戦闘に参加します。<br/>";
                    } else {
                    msg += this.fleet[i].getName() + "は航宙L不足で戦闘に参加できません。<br/>";
                }
            } else {
                msg += this.fleet[i].getName() + "は戦闘に参加します。<br/>";
                list.push(this.fleet[i]);
            }
        }
        if (msg != "") {
            printConsole(this.id, msg);
        }

        return list;
    }

    // 全ての星に到達可能かどうかを判定する。
    this.isReachAllPlanets = function() {
        if (this.ctlPlanetList.length <= 1) {
            return true;
        }
        var sp = this.ctlPlanetList[0];
        var starList = this.ctlPlanetList;

        var starHash = new Object();
        for (var i = 0; i < starList.length; i++) {
            starHash[sp.loc] = false;
        }

        var clist = new Array();
        clist.push(sp.loc);
        if (starHash[sp.loc] == 0) {
            starHash[sp.loc] = true;
        }
        while(clist.length > 0) {
            var nlist = new Array();
            for (var i = 0; i < clist.length; i++) {
                var cloc = clist[i];
                for (var lk in starHash) {
                    if (starHash[lk]) {
                        continue;
                    }
                    if (calDistance(cloc, lk) <= this.nvg) {
                        starHash[lk] = true;
                        nlist.push(lk);
                    }
                }
            }
            clist = nlist;
        }

        for (var lk in starHash) {
            if (! starHash[lk]) {
                return false;
            }
        }

        return true;
    }

    this.calTotalMaxEcn = function() {
        var result = 0;
        for (var i = 0; i < this.ctlPlanetList.length; i++) {
            var p = this.ctlPlanetList[i];
            result += p.calMaxEcn();
        }
        return result;
    }

    return this;
}
